@@ -5,8 +5,16 @@ use Plack::Util::Accessor qw(timeout response);
 use Plack::Request;
 use Plack::Response;
 use Scope::Guard ();
+use Time::HiRes qw(alarm time);
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
+
+sub _get_alarm_subref {
+    my $self = shift;
+
+   my $timeout = $self->timeout || 120;
+   return ref($timeout) eq 'CODE' ? $timeout : sub { alarm($timeout) };
+}
 
 sub call {
     my ( $self, $env ) = @_;
@@ -18,7 +26,8 @@ sub call {
     eval {
 
         $time_started = time();
-        alarm( $self->timeout || 120 );
+        $self->_get_alarm_subref->();
+
         my $guard = Scope::Guard->new(sub {
             alarm 0;
         });
@@ -27,16 +36,16 @@ sub call {
 
     } or do {
         my $request        = Plack::Request->new($env);
-        my $execution_time = time() - $time_started;
-        warn sprintf
-          "Terminated request for uri '%s' - took %d seconds (timeout %ds)",
-          $request->request_uri,
-          $execution_time,
-          $self->timeout;
 
         my $response = Plack::Response->new(408);
         if ( my $build_response_coderef = $self->response ) {
-            $build_response_coderef->($response);
+            $build_response_coderef->($response,$execution_time);
+        }
+        else {
+            # warn by default, so there's a trace of the timeout left somewhere
+            warn sprintf
+              "Terminated request for uri '%s' due to timeout (%ds)",
+              $self->timeout;
         }
 
         return $response->finalize;
@@ -57,15 +66,17 @@ Plack::Middleware::Timeout
 
     Plack::Middleeare::Timeout->wrap(
         $app,
-        timeout  => 60,
+        timeout  => sub { ... } || 60,
         # optional callback to set the custom response 
-            my $plack_response = shift;
+        response => sub {
+            my ($response_obj,$execution_time) = @_;
 
-            $response->code(HTTP_REQUEST_TIMEOUT);
-            $response->body( encode_json({
+            $response_obj->code(HTTP_REQUEST_TIMEOUT);
+            $response_obj->body( encode_json({
                 timeout => 1,
                 other_info => {...},
             }));
+
             return $plack_response;
         }
     );
@@ -74,6 +85,27 @@ Plack::Middleware::Timeout
 
 Timeout any plack requests at an arbitrary time.
 
+=head1 PARAMETERS
+
+=over
+
+=item timeout
+
+Determines how we're going to get the timeout value, either subref returning a value or any value accepted by subroutine defined in Time::HiRes::alarm, default 120 seconds.
+
+=item response
+
+Optional subroutine that receives two parameters, a Plack::Response object that we can further modify 
+to fit our needs, if none is provided the middleware resolves to emitting a warning:
+
+=over
+
+'Terminated request for uri '%s' due to timeout (%ds)'
+
+=back
+
+=back
+
 =head1 AUTHOR
 
 Tomasz Czepiel <tjmc@cpan.org>
